import java.io.FileReader;
import java_cup.runtime.Symbol;
import java.io.FileWriter;
import java.io.IOException;

class CompilerMain {

    static public void main(String[] args) throws Exception{
      
        String inputFileName = null;
        String outputFileName = null;
        boolean isDebugOn = false;

        if(args.length == 2){
            inputFileName = args[0];
            outputFileName = args[1];
        } else if(args.length == 3 && args[0].equals("-debug")){
            inputFileName = args[1];
            outputFileName = args[2];
            isDebugOn = true;
        } else {
            System.err.println("Usage: java -jar Compiler.jar [-debug] inputfile outputfile");
            System.exit(1);
        }

        Parser parser = new Parser(new Yylex(new FileReader(inputFileName)));

        Symbol parseTree;

        if(isDebugOn)
            parseTree = parser.debug_parse();
        else
            parseTree = parser.parse();

        Procedure mainProc = (Procedure) parseTree.value;

        try {
            FileWriter output = new FileWriter(outputFileName);
            output.write(mainProc.generate());
            output.write(libraryCode);
            output.close();
        } catch(IOException e){
            System.err.println("I/O exception: " + e);
            System.exit(1);
        }
    }

    private static String libraryCode =
        ".data\n" +
        "print_int_newline:\n" +
        ".asciiz \"\\n\"\n" +
        ".text\n" +
        "print_int:\n" +
        "li $v0, 1\n" +
        "syscall\n" +
        "la $a0, print_int_newline\n" +
        "li $v0, 4\n" +
        "syscall\n" +
        "jr $ra\n";
}
